"""Console script for pyvue."""
import os, sys, zipfile, shutil, requests
import socket, errno
import sqlite3
from .static import templateZip
from .logger import Logger

def main(args):
    stateFile = os.path.join(os.getcwd(), ".state")
    """Console script for pyvue."""
    if args["job"] == "init":
        Logger.info("Creating pyvuejs application...")
        if args["app"] == "":
            Logger.info("Please input AppName")
            args["app"] = input("AppName: ")

        appDir = os.path.join(os.getcwd(), args["app"])
        if os.path.exists(appDir):
            raise RuntimeError("App {} already exists!".format(args["app"]))
        else:
            os.mkdir(appDir)

        Logger.info("Extracting template files...")
        zp = zipfile.ZipFile(templateZip)
        zp.extractall(appDir)
        zp.close()

        Logger.info("App \"{}\" is ready!".format(args["app"]))

    elif args["job"] == "run":
        dirList = os.listdir(os.getcwd())
        if os.path.exists(stateFile):
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                sock.bind(("127.0.0.1", int(args["port"])))
                os.remove(stateFile)
            except:
                raise RuntimeError("Application is already started!")

        if "manage.py" in dirList and "views" in dirList:
            con = sqlite3.connect(stateFile, check_same_thread = False)
            cursor = con.cursor()
            cursor.execute("create table `state` (`PORT` INT);")
            cursor.execute("insert into `state` values ({});".format(args["port"]))
            con.commit()
            cursor.close()
            con.close()

            if args["mode"] == "server":
                Logger.info("Starting pyvuejs application...")
                if not "static" in dirList:
                    Logger.info("Static files are missing!")

                from .server import Server
                Server(os.getcwd(), args["logging"] == "enable").start(args["host"], int(args["port"]))
            elif args["mode"] == "standalone":
                window_size = [int(item) for item in args["window_size"].split(",")]
                if len(window_size) < 2:
                    window_size = [window_size[0], window_size[0]]
                elif len(window_size) == 0:
                    window_size = [900, 600]

                from .server import WindowedServer
                WindowedServer(args["logging"] == "enable").start(os.getcwd(), int(args["port"]), window_size)
            else:
                raise RuntimeError("Unknown mode {}, available modes are \"server\", \"standalone\"".format(args["mode"]))
        else:
            raise RuntimeError("Required files are missing! Please check \"manage.py\" file and \"views\" directory!")

    elif args["job"] == "stop":
        if os.path.exists(stateFile):
            con = sqlite3.connect(stateFile, check_same_thread = False)
            cursor = con.cursor()

            requests.post("http://127.0.0.1:{}/shutdown".format(
                cursor.execute("select `port` from `state`;").fetchone()[0]
            ))
            
            cursor.close()
            con.close()

            os.remove(stateFile)

    elif args["job"] == "create":
        Logger.info("Creating {0} {1}...".format(args["type"], args["name"]))
        if args["type"] == "plugin":
            targetDir = os.path.join(os.getcwd(), "plugins", args["name"])
            if not os.path.exists(targetDir):
                os.mkdir(targetDir)
                with open(os.path.join(targetDir, "__init__.py"), "w", encoding = "utf-8") as initW:
                    initW.write("# -*- coding: utf-8 -*-\n")

                Logger.info("Plugin {0} is ready!".format(args["name"]))
            else:
                Logger.warn("Plugin {0} already exists!".format(args["name"]))
        elif args["type"] == "folder":
            targetDir = os.path.join(os.getcwd(), args["name"])
            if not os.path.exists(targetDir):
                os.mkdir(targetDir)
                Logger.info("Folder {0} is created!".format(args["name"]))
            else:
                Logger.warn("Folder {0} already exists!".format(args["name"]))
        elif args["type"] == "file":
            targetFile = os.path.join(os.getcwd(), args["name"])
            if not os.path.exists(targetFile):
                with open(targetFile, "w", encoding = "utf-8") as fileW:
                    fileW.write("")

                Logger.info("File {0} is created!".format(args["name"]))
            else:
                Logger.warn("Fild {0} already exists!".format(args["name"]))

    elif args["job"] == "remove":
        Logger.info("Removing {0} {1}...".format(args["type"], args["name"]))
        if args["type"] in ("plugin", "folder"):
            targetDir = os.path.join(os.getcwd(), "plugins", args["name"]) if args["type"] == "plugin" else os.path.join(os.getcwd(), args["name"])
            if os.path.exists(targetDir):
                shutil.rmtree(targetDir)
                Logger.info("{0} {1} is removed!".format(args["type"].capitalize(), args["name"]))
        elif args["type"] == "file":
            targetFile = os.path.join(os.getcwd(), args["name"])
            if os.path.exists(targetFile):
                os.remove(targetFile)
                Logger.info("File {0} is removed!".format(args["name"]))

    elif args["job"] == "build":
        raise RuntimeError("Build is not available now!")

if __name__ == "__main__":
    sys.exit(main(sys.argv))  # pragma: no cover



# -*- coding: utf-8 -*-

class Binder():
    def __init__(self):
        self.__variables = []
        self.__methods = []
        self.__computes = []
        self.__events = {}

    @property
    def variables(self) -> list:
        return self.__variables

    @property
    def methods(self) -> list:
        return self.__methods

    @property
    def computes(self) -> list:
        return self.__computes

    @property
    def events(self) -> dict:
        return self.__events

    def variable(self, variable) -> bool:
        if not variable in self.__variables:
            self.__variables.append(variable)
            return True
        else:
            return False

    def method(self, func):
        def decorator(func):
            if not func.__name__ in self.__methods:
                self.__methods.append(func.__name__)

            return func

        return decorator(func)

    def compute(self, func):
        def decorator(func):
            if not func.__name__ in self.__computes:
                self.__computes.append(func.__name__)

            return func

        return decorator(func)

    def event(self, eventType:str):
        def decorator(func):
            self.__events[eventType] = func.__name__

            return func

        return decorator

class Model():
    binder = Binder()
    method = binder.method
    compute = binder.compute
    event = binder.event

    def __init__(self, webview = None):
        from copy import deepcopy

        self.__webview = webview

        excludeList = ["binder", "session", "method", "compute", "event", "name", "webview", "variables", "sessions", "computes", "methods", "events"]
        self.__mayVariables = [mname for mname in dir(self) if not mname in excludeList and not mname.startswith("__") and not mname.startswith("_Model__")]
        self.__sessions = {}
        for varName in self.__mayVariables:
            if varName.startswith("session_"):
                varNameVisible = varName[8:]
                exec("self.{0} = deepcopy(self.{1})".format(varNameVisible, varName))

                self.__mayVariables.remove(varName)
                # self.__mayVariables.append(varNameVisible)

                self.__sessions[varNameVisible] = eval("self.{}".format(varNameVisible))

    @property
    def name(self) -> str:
        return self.__class__.__name__

    @property
    def webview(self):
        return self.__webview

    @property
    def variables(self) -> dict:
        variableInfo = {}
        for varName in self.__mayVariables:
            if not varName in self.binder.computes and not varName in self.binder.methods and not varName in self.__sessions.keys() and not varName in self.binder.events.values():
                variableInfo[varName] = eval("self.{}".format(varName))

        return variableInfo

    @property
    def sessions(self) -> dict:
        return self.__sessions

    @property
    def computes(self) -> dict:
        computeInfo = {}
        for computeName in self.binder.computes:
            computeInfo[computeName] = eval("self.{}".format(computeName))

        return computeInfo

    @property
    def methods(self) -> dict:
        methodInfo = {}
        for methodName in self.binder.methods:
            methodInfo[methodName] = eval("self.{}".format(methodName))

        return methodInfo

    @property
    def events(self) -> dict:
        eventInfo = {}
        for eventType, eventName in self.binder.events.items():
            eventInfo[eventType] = eval("self.{}".format(eventName))

        return eventInfo

class View():
    def __init__(self, name:str, prefix:str, resourceText:str, styleText:str, scriptText:str, templateText:str, modelTextInfo:dict, webview = None):
        from .static import baseView

        self.__name = name
        self.__prefix = prefix

        self.__renderedText = baseView.replace(
            "{$viewName}", self.__name
        ).replace(
            "{$viewModels}", "\", \"".join(list(modelTextInfo.keys()))
        ).replace(
            "{$viewResource}", resourceText
        ).replace(
            "{$viewStyle}", styleText
        ).replace(
            "{$viewScript}", scriptText
        ).replace(
            "{$viewBody}", templateText
        )
        # if self.__prefix == "view":
        #     for line in self.__renderedText.split("\n"):
        #         if line.strip().startswith("<component ") and "name" in line:
        #             componentName = line[11:-1].split("=")[1][1:-1]

        #             self.__renderedText = self.__renderedText.replace(
        #                 line,
        #                 '<div style="width:100%;height:100%;"><object type="text/html" data="/components/{}" style="overflow:hidden;width:100%;height:100%;"></object></div>'.format(componentName)
        #             )

        self.__models = {}
        for modelName, modelText in modelTextInfo.items():
            exec(modelText)
            self.__models[modelName] = eval(modelName)(webview)

    @property
    def name(self) -> str:
        return self.__name

    @property
    def prefix(self) -> str:
        return self.__prefix

    @property
    def models(self) -> dict:
        return self.__models

    def render(self, viewId:str) -> str:
        return self.__renderedText.replace("{$viewId}", viewId)



import sys, argparse
from .cli import main

args = argparse.ArgumentParser(description = "manage script of pyvuejs")
args.add_argument("job", help = "job to run app")
# appName(init)
args.add_argument("--app", default = "", help = "[init] name of app")
# host, port(run)
args.add_argument("--host", default = "0.0.0.0", help = "[run] host of backend server")
args.add_argument("--port", default = 8000, help = "[run] port of backend server")
args.add_argument("--logging", default = "enable", help = "[run] choose if logging to console")
args.add_argument("--mode", default = "server", help = "[run] mode of application")
args.add_argument("--window-size", default = "900,600", help = "[run] size of standalone window")
# type, name(create)
args.add_argument("--type", default = "plugin", help = "[create, remove] type of target(plugin, folder, file)")
args.add_argument("--name", default = "plugin1", help = "[create, remove] name of target")
# dist(build)
args.add_argument("--dist", default = "dist", help = "[build] build dist file location")

if __name__ == "__main__":
    try:
        sys.exit(main(vars(args.parse_args())))
    except KeyboardInterrupt:
        pass



# -*- coding: utf-8 -*-
import os
from . import __path__

baseView = """
<!DOCTYPE HTML5>
<html>
    <head>
        <link rel="shortcut icon" href="/static/favicon.ico">
        <link rel="icon" href="/static/favicon.png">
        <link rel="shortcut icon" href="/app/favicon.ico">
        <link rel="icon" href="/app/favicon.png">
        <script type="text/javascript" src="/static/socket.io.dev.js"></script>
        <script type="text/javascript" src="/static/vue.min.js"></script>
        <script type="text/javascript" src="/static/pyvuejs.js"></script>
        {$viewResource}
        <style>
            {$viewStyle}
        </style>
        <script>
            {$viewScript}
        </script>
    </head>
    <body style="width:100vw;height:100vh;margin:0px auto;overflow:hidden;">
        {$viewBody}

        <script>
            (new pyvuejs("{$viewId}", "{$viewName}", "view", ["{$viewModels}"])).init();
        </script>
    </body>
</html>
"""
templateZip = os.path.join(__path__[0], "template.zip")

del __path__
del os



# -*- coding: utf-8 -*-
import logging

Logger = logging.Logger("pyvuejs")

__handler = logging.StreamHandler()
__handler.setFormatter(logging.Formatter("[%(name)s | %(asctime)s] %(levelname)s: %(message)s", datefmt = "%Y-%m-%dT%H:%M:%SZ"))
Logger.addHandler(__handler)

del logging



# -*- coding: utf-8 -*-
# import modules
import os, sys, json
from threading import Timer
from collections import OrderedDict
from datetime import datetime
from glob import glob
from copy import copy
import subprocess
from flask import Flask, Blueprint, redirect, request, jsonify
from flask_socketio import SocketIO, emit

from . import __path__
from .models import View
from .logger import Logger

class Server():
    def __init__(self, appDir:str, enableLogging:bool = True, webview = None):
        # check logging
        self.__logging = enableLogging

        # variables
        if self.__logging:
            Logger.info("Prepare Server to run app...")

        self.__webview = webview
        self.__appDir = os.path.abspath(appDir)
        self.__session = {
            "app": {
                "mpa": {},
                "views": {},
                "components": {},
                "updatedTime": {}
            },
            "data": {
                "mpa": {},
                "excludes": { "id": "", "name": "" }
            },
            "config": {
                "expiredTime": 1800
            }
        }
        self.__app = Flask(
            "pyvue",
            static_folder = os.path.join(__path__[0], "static"),
            template_folder = os.path.join(__path__[0], "static")
        )
        self.__app.secret_key = "pyvuejsApp"
        self.__socketio = SocketIO(self.__app)

        # view routing points
        if self.__logging:
            Logger.info("Setting view/component routing points...")

        @self.__app.route("/views/<viewName>")
        def showView(viewName):
            if viewName in self.__session["app"]["views"].keys():
                viewId = request.remote_addr
                if not viewId in self.__session["data"]["mpa"].keys():
                    self.__session["data"]["mpa"][viewId] = {}

                if not viewId in self.__session["app"]["mpa"].keys():
                    self.__session["app"]["mpa"][viewId] = OrderedDict()

                if not viewName in self.__session["app"]["mpa"][viewId].keys():
                    self.__session["app"]["mpa"][viewId][viewName] = copy(self.__session["app"]["views"][viewName])

                    for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
                        for varName, var in model.sessions.items():
                            self.__session["data"]["mpa"][viewId][varName] = var

                        if "load" in model.events.keys():
                            model.events["load"](self.__session["data"]["mpa"][viewId])

                for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
                    if "show" in model.events.keys():
                        model.events["show"](self.__session["data"]["mpa"][viewId])

                return self.__session["app"]["mpa"][viewId][viewName].render(viewId)
            else:
                viewNames = list(self.__session["app"]["views"].keys())
                if viewName == "def@ltWindowed" and len(viewNames) > 0:
                    if "main" in viewNames:
                        return redirect("/views/main")
                    else:
                        return redirect("/views/{}".format(viewNames[0]))
                else:
                    return ""

        @self.__app.route("/components/<viewName>")
        def showComponent(viewName):
            if viewName in self.__session["app"]["components"].keys():
                viewId = request.remote_addr
                if not viewId in self.__session["data"]["mpa"].keys():
                    self.__session["data"]["mpa"][viewId] = {}

                if not viewId in self.__session["app"]["mpa"].keys():
                    self.__session["app"]["mpa"][viewId] = OrderedDict()

                if not viewName in self.__session["app"]["mpa"][viewId].keys():
                    self.__session["app"]["mpa"][viewId][viewName] = copy(self.__session["app"]["components"][viewName])

                    for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
                        for varName, var in model.sessions.items():
                            self.__session["data"]["mpa"][viewId][varName] = var

                        if "load" in model.events.keys():
                            model.events["load"](self.__session["data"]["mpa"][viewId])

                for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
                    if "show" in model.events.keys():
                        model.events["show"](self.__session["data"]["mpa"][viewId])

                return self.__session["app"]["mpa"][viewId][viewName].render(viewId)
            else:
                return ""

        # function routing points
        if self.__logging:
            Logger.info("Setting function routing points...")

        @self.__app.before_first_request
        def onFirstRequest():
            pass

        @self.__app.route("/shutdown", methods = ["POST"])
        def onShutdown():
            if self.__logging:
                Logger.info("App is shutting down...")

            sys.exit()

        @self.__app.route("/session/upload", methods = ["POST"])
        def onSessionUpload():
            res = json.loads(request.data)

            self.__session["data"]["mpa"][res["id"]] = res["session"]
            self.__session["data"]["excludes"]["id"] = res["id"]
            self.__session["data"]["excludes"]["name"] = res["name"]
            self.__session["app"]["updatedTime"][res["id"]] = datetime.now()

            return jsonify({
                "state": "success"
            })

        @self.__app.route("/session/download", methods = ["POST"])
        def onSessionDownload():
            res = json.loads(request.data)

            try:
                # check expiredTime
                for viewId, updateTime in self.__session["app"]["updatedTime"].items():
                    if not res["id"] == viewId:
                        if (datetime.now() - updateTime).total_seconds() >= self.__session["config"]["expiredTime"]:
                            self.__session["app"]["mpa"].pop(viewId)
                            self.__session["data"]["mpa"].pop(viewId)

                resp = {
                    "id": self.__session["data"]["excludes"]["id"],
                    "excludeName": self.__session["data"]["excludes"]["name"],
                    "sessions": {}
                }
                for view in self.__session["app"]["mpa"][resp["id"]].values():
                    sessionSet = {}
                    for modelName in view.models.keys():
                        sessionSet[modelName] = self.__session["data"]["mpa"][resp["id"]]

                    resp["sessions"][view.name] = sessionSet

                return jsonify(resp)
            except KeyError:
                return jsonify({
                    "id": "",
                    "excludeName": "",
                    "sessions": {}
                })

        # socket points
        if self.__logging:
            Logger.info("Setting socket routing points...")

        @self.__socketio.on("feedbackPY", namespace = "/pyvuejsSocket")
        def onFeedback(res):
            print(res)

        @self.__socketio.on("initViewPY", namespace = "/pyvuejsSocket")
        def onInitView(res):
            if res["id"] in self.__session["app"]["mpa"].keys():
                view = self.__session["app"]["mpa"][res["id"]][res["name"]]
                emit("initViewJS", {
                    "id": res["id"],
                    "name": res["name"],
                    "session": self.__session["data"]["mpa"][res["id"]],
                    "data": {
                        modelName: model.variables
                        for modelName, model in view.models.items()
                    },
                    "computes": {
                        modelName: list(model.computes.keys())
                        for modelName, model in view.models.items()
                    },
                    "methods": {
                        modelName: list(model.methods.keys())
                        for modelName, model in view.models.items()
                    }
                })

        @self.__socketio.on("compute", namespace = "/pyvuejsSocket")
        def onCompute(res):
            model = self.__session["app"]["mpa"][res["id"]][res["name"]].models[res["model"]]

            for varName, value in res["variables"].items():
                exec("model.{} = value".format(varName))

            self.__session["data"]["mpa"][res["id"]] = res["session"]
            model.computes[res["method"]](self.__session["data"]["mpa"][res["id"]])
            
            emit("update", {
                "id": res["id"],
                "name": res["name"],
                "model": model.name,
                "variables": model.variables,
                "session": self.__session["data"]["mpa"][res["id"]]
            })

        @self.__socketio.on("method", namespace = "/pyvuejsSocket")
        def onMethod(res):
            model = self.__session["app"]["mpa"][res["id"]][res["name"]].models[res["model"]]

            for varName, value in res["variables"].items():
                exec("model.{} = value".format(varName))

            self.__session["data"]["mpa"][res["id"]] = res["session"]
            model.methods[res["method"]](self.__session["data"]["mpa"][res["id"]])
            
            emit("update", {
                "id": res["id"],
                "name": res["name"],
                "model": model.name,
                "variables": model.variables,
                "session": self.__session["data"]["mpa"][res["id"]]
            })

        if self.__logging:
            Logger.info("Routing points are ready!\n")

        self.__interpret_appDir()
        if self.__logging:
            Logger.info("Server is ready!\n")

    def __interpret_appDir(self):
        if self.__logging:
            Logger.info("Interpreting app...")

        if os.path.exists(self.__appDir):
            sys.path.append(self.__appDir)

            if self.__logging:
                Logger.info("Interpreting view files...")

            for pvPath in glob(os.path.join(self.__appDir, "views", "*.pvue")):
                pvInfo = self.__interpret_pyvue(pvPath)
                
                view:View = View(
                    pvInfo["name"], pvInfo["prefix"],
                    pvInfo["resource"], pvInfo["style"], pvInfo["script"],
                    pvInfo["template"],
                    pvInfo["model"],
                    self.__webview
                )

                if pvInfo["prefix"] == "view":
                    self.__session["app"]["views"][pvInfo["name"]] = view
                elif pvInfo["prefix"] == "component":
                    self.__session["app"]["components"][pvInfo["name"]] = view

                if self.__logging:
                    Logger.info("{}.pvue has been interpreted".format(pvInfo["name"]))

            if self.__logging:
                Logger.info("Finished!\n")

            if self.__logging:
                Logger.info("Linking static files to server...")
            staticDir = os.path.join(self.__appDir, "static")
            if os.path.exists(staticDir):
                self.__app.register_blueprint(
                    Blueprint(
                        os.path.basename(self.__appDir), "pyvue",
                        static_url_path = "/app",
                        static_folder = staticDir
                    )
                )
            if self.__logging:
                Logger.info("Finished!\n")
        
        if self.__logging:
            Logger.info("App has been ready!\n")

    def __interpret_pyvue(self, pvFile) -> dict:
        import os

        with open(pvFile, "r", encoding = "utf-8") as pvr:
            pvt = pvr.read()

            prefixLine = pvt.split("\n")[0]
            if prefixLine.startswith("!prefix"):
                prefix = prefixLine[8:]
            else:
                prefix = "view"

            pvInfo = {
                "resource": "",
                "style": "",
                "template": "",
                "script": "",
                "model": {}
            }
            for key in pvInfo.keys():
                try:
                    block = pvt.split("<{}>".format(key))[1].split("</{}>".format(key))[0]
                except IndexError:
                    block = ""

                if not block == "":
                    lines = block.split("\n")[1:-1]
                    if len(lines) > 0:
                        stripLength = len(lines[0][:-1 * len(lines[0].strip())])

                        blockStripLines = [
                            line[stripLength:]
                            for line in lines
                        ]

                        if key == "template":
                            if prefix == "view":
                                for idx in range(len(blockStripLines)):
                                    line = blockStripLines[idx]
                                    if line.strip().startswith("<component ") and "name" in line:
                                        componentName = line[11:-2].split("=")[1].strip()[1:-1]
                                        tabSpace = line.replace(line.strip(), "")

                                        blockStripLines[idx] = tabSpace + '<object type="text/html" data="/components/{}" style="overflow:hidden;width:100%;height:100%;"></object>'.format(componentName)

                            pvInfo[key] = "\n".join(blockStripLines)
                        elif key == "model":
                            lineNums = []
                            for idx in range(len(blockStripLines)):
                                line = blockStripLines[idx]
                                if line.startswith("Model ") and line.endswith(":"):
                                    lineNums.append(idx)
                                    blockStripLines[idx] = "class {}(Model):".format(line[6:-1])
                                elif line.strip().startswith("def ") and line.endswith(":"):
                                    blockStripLines[idx] = line.split("(self")[0] + "(self, session):"


                            for idx in range(len(lineNums)):
                                startNum = lineNums[idx]
                                if idx < len(lineNums) - 1:
                                    endNum = lineNums[idx + 1]
                                else:
                                    endNum = len(blockStripLines)

                                modelLines = blockStripLines[startNum:endNum]
                                if modelLines[-1] == "":
                                    modelLines.pop(-1)

                                for mlIdx in range(1, len(modelLines)):
                                    line = modelLines[mlIdx]
                                    if ":session" in line:
                                        varName = line.split(":session")[0].strip()
                                        modelLines[mlIdx] = line.replace(
                                            varName + ":session",
                                            "session_" + varName
                                        )

                                insertSpace = modelLines[1].replace(modelLines[1].strip(), "")
                                modelLines.insert(1, insertSpace + "binder = Binder()")
                                modelLines.insert(2, insertSpace + "method = binder.method")
                                modelLines.insert(3, insertSpace + "compute = binder.compute")
                                modelLines.insert(4, insertSpace + "event = binder.event")
                                modelLines.insert(5, "")

                                pvInfo[key][modelLines[0][6:-8]] = "\n".join(modelLines)
                        else:
                            pvInfo[key] = "\n".join(blockStripLines)

            pvInfo["name"] = os.path.splitext(os.path.basename(pvFile))[0]
            pvInfo["prefix"] = prefix

            return pvInfo

    def __get_view_type(self, name:str) -> str:
        if name in self.__session["app"]["views"].keys():
            return "view"
        elif name in self.__session["app"]["components"].keys():
            return "component"

    def __create_viewFunc(self, fName, view):
        return """
def {0}():
    return '''
{1}
'''
""".format(fName, view.render())

    def start(self, host:str = "0.0.0.0", port:int = 8080):
        try:
            if host in ("127.0.0.1", "localhost"):
                host = "0.0.0.0"

            if self.__logging:
                Logger.info("Server started on \"http://{}:{}\"".format(host, port))
                Logger.info("Please check Devtool to show data transfers")
            self.__socketio.run(self.__app, host = host, port = port)
        except KeyboardInterrupt:
            os.remove(os.path.join(self.__appDir, ".state"))

class WindowedServer():
    def __init__(self, enableLogging:bool = True):
        from PySide2.QtWidgets import QApplication
        from PySide2.QtWebEngineWidgets import QWebEngineView

        self.__logging = enableLogging

        self.__qtApp = QApplication(sys.argv)
        self.__window = QWebEngineView()
        self.__window.loadFinished.connect(self.__onLoaded)
        self.__window.closeEvent = self.__onClosed

    def __onLoaded(self, finished:bool):
        self.__window.showNormal()

        if self.__logging:
            Logger.info("Webview is loaded")

    def __onClosed(self, ev):
        if self.__logging:
            Logger.info("Shutting down background server...")

        subprocess.Popen([sys.executable, ".\manage.py", "stop"])

    def __start_server_background(self, appDir:str, port:int):
        Server(appDir, False, self.__window).start("127.0.0.1", port)

    def start(self, appDir:str, port:int = 8080, window_size:list = [900, 600]):
        from PySide2.QtGui import QIcon
        from PySide2.QtCore import QUrl, QThread

        if self.__logging:
            Logger.info("Start server on background...")

        os.chdir(appDir)
        self.__timer = Timer(1, self.__start_server_background, (appDir, port))
        self.__timer.start()
        # subprocess.Popen([sys.executable, ".\manage.py", "run", "--host={}".format(host), "--port={}".format(port), "--logging=disable", "--mode=server"])

        if self.__logging:
            Logger.info("Setting up webview...")

        appIcon = os.path.join(appDir, "static", "favicon.png")
        if not os.path.exists(appIcon):
            appIcon = os.path.join(__path__[0], "static", "favicon.png")

        self.__qtApp.setWindowIcon(QIcon(appIcon))

        self.__window.setWindowTitle(os.path.basename(appDir))
        self.__window.load(QUrl("http://127.0.0.1:{}/views/def@ltWindowed".format(port)))
        self.__window.resize(window_size[0], window_size[1])

        sys.exit(self.__qtApp.exec_())
